==> OpenXOPS-tags_v1.020-20150628-r97.tar.gz: OpenXOPS-tags_v1.020-20150628-r97/v1.020-20150628/window.cpp <==
//! @file window.cpp
//! @brief CEBhE̍쐬E

//--------------------------------------------------------------------------------
// 
// OpenXOPS
// Copyright (c) 2014-2015, OpenXOPS Project / [-_-;](mikan) All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice, 
//   this list of conditions and the following disclaimer in the documentation 
//   and/or other materials provided with the distribution.
// * Neither the name of the OpenXOPS Project nor the@names of its contributors 
//   may be used to endorse or promote products derived from this software 
//   without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL OpenXOPS Project BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//--------------------------------------------------------------------------------

#include "window.h"

//! @brief RXgN^
WindowControl::WindowControl()
{
	hInstance = 0;
	nCmdShow = 0;
	hWnd = NULL;
}

//! @brief fBXgN^
WindowControl::~WindowControl()
{
	//
}

//! @brief AvP[V̏ݒ
//! @param in_hInstance CX^X nh
//! @param in_nCmdShow EBhE̕\
void WindowControl::SetParam(HINSTANCE in_hInstance, int in_nCmdShow)
{
	hInstance = in_hInstance;
	nCmdShow = in_nCmdShow;
}

//! @brief CEBhE쐬
//! @param title EBhE^Cg
//! @param width EBhE̕
//! @param height EBhE̍
//! @param fullscreen falseFEBhE\@trueFtXN[p\
//! @return Ftrue@sFfalse
//! @warning SetParam()֐Őݒ肵ĂKv܂B
bool WindowControl::InitWindow(char* title, int width, int height, bool fullscreen)
{
	WNDCLASS wc;
	int x, y;
	RECT Rect;
	DWORD dwStyle;

	//EBhENX̓o^
	wc.style		= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc	= WindowProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInstance;
	wc.hIcon		= LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(MAINICON));
	wc.hCursor		= NULL;
	wc.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName	= NULL;
	wc.lpszClassName	= "MainWindow";
	if( !RegisterClass(&wc) ){
		return false;
	}

	if( fullscreen == false ){
		dwStyle = WS_OVERLAPPED | WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX;
	}
	else{
		dwStyle = WS_POPUP;
	}

	//EBhETCYvZ
	Rect.left = 0;
	Rect.top = 0;
	Rect.right = width;
	Rect.bottom = height;
	AdjustWindowRect(&Rect, dwStyle, FALSE);
	width = Rect.right - Rect.left;
	height = Rect.bottom - Rect.top;

	//\ʒu
	x = 0;
	y = 0;

	/*
	//EChE[hȂAEChEʂ̒Sɔzu
	if( fullscreen == false ){
		SystemParametersInfo(SPI_GETWORKAREA, 0, &Rect, 0);
		x = (Rect.right - width)/2;
		y = (Rect.bottom - height)/2;
	}
	*/

	//EBhE쐬
	hWnd = CreateWindow( "MainWindow", title, dwStyle, x, y, width, height, NULL, NULL, hInstance, NULL );
	if( hWnd == NULL ){
		return false;
	}

	//\
	ShowWindow( hWnd, nCmdShow );

	return true;
}

//! @brief CEBhẼEBhEvV[W
LRESULT WINAPI WindowControl::WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg ){
		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//! @brief EBhEnh擾
//! @return EBhEnh
HWND WindowControl::GethWnd()
{
	return hWnd;
}

//! @brief EChEbZ[WiC[vj
//! @return AChԁF0@Q[̃CsF1@EChEF-1
//! @attention {֐u1vԂꂽꍇɌAQ[̃CsĂB
//! @attention {֐u-1vԂꂽꍇAe\[XC^[tF[XAQ[IĂB
int WindowControl::CheckMainLoop()
{
	MSG msg = {0};

	if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) ){
		//EChEbZ[WĂ

		TranslateMessage( &msg );
		DispatchMessage( &msg );

		if( msg.message == WM_QUIT ){
			return -1;
		}
		return 0;
	}
	else if( GetActiveWindow() == hWnd ){
		//Q[̃t[
		return 1;
	}
	//else{
		//EChEbZ[W܂ő҂
		WaitMessage();
		return 0;
	//}
}

//! @brief G[bZ[W\
//! @param *str bZ[W
void WindowControl::ErrorInfo(char *str)
{
	MessageBox(hWnd, str, "ERROR", MB_OK);
}

//! @brief EBhE悤ɗv
void WindowControl::CloseWindow()
{
	PostMessage(hWnd, WM_CLOSE, 0L, 0L);
}

//! @brief fpsiFrames Per SecondFt[[gjvZ
//! @param getcnt fps擾it[Pʁj
//! @return fps
float GetFps(int getcnt)
{
	static unsigned int ptimeg = 0;
	unsigned int nowtime;
	static float pfps = 0.0f;
	static int cnt = 0;

	if( cnt == 0 ){
		ptimeg = GetTimeMS();
	}
	if( cnt == getcnt ){
		nowtime = GetTimeMS();
		pfps = 1000.0f / ((nowtime - ptimeg)/getcnt);
		cnt = 0;
	}
	else{
		cnt += 1;
	}

	return pfps;
}

//! @brief fpsiFrames Per SecondFt[[gj
//! @return {Ftrue@@{Ffalse
bool ControlFps()
{
	static unsigned int ptimec = 0;
	unsigned int nowtime;
	int waittime;

	nowtime = GetTimeMS();
	waittime = GAMEFRAMEMS - (nowtime - ptimec);
	if( waittime > 0 ){
		Sleep(waittime);
		ptimec = GetTimeMS();
		return true;
	}

	ptimec = nowtime;
	return false;
}

//! @brief ~bPʂԂ
//! @return ~b
unsigned int GetTimeMS()
{
	unsigned int time;

	timeBeginPeriod(1);
	time = timeGetTime();
	timeEndPeriod(1);

	return time;
}

//! @brief ɂ镶擾
//! @param str 󂯎|C^
void GetTimeName(char *str)
{
	time_t timer;
	struct tm *local;

	timer = time(NULL);
	local = localtime(&timer);

	sprintf(str, "%04d%02d%02d%02d%02d%02d", local->tm_year + 1900, local->tm_mon + 1, local->tm_mday, local->tm_hour, local->tm_min, local->tm_sec);
}

//! @brief 
void InitRand()
{
	srand(GetTimeMS());
}

//! @brief _ȐlԂ
//! @param num ͈
//! @return 0`num-1
int GetRand(int num)
{
	return rand()%num;

	//return rand() / (RAND_MAX/num);

	//{XOPS̃ASYH
	//static int memory = GetTimeMS();
	//int x;
	//memory = memory * 214013 + 2745024;
	//x = memory >> 16;
	//x = x & 0x00007FFF;
	//return x%num;
}

#ifdef PATH_DELIMITER_SLASH
//! @brief pX؂蕶ϊ
//! @param *str t@CpX̃|C^
//! @return Vt@CpX̃|C^
//! @note '\''/'֒u܂B
char* ChangePathDelimiter(char *str)
{
	static char newstr[MAX_PATH];
	strcpy(newstr, str);

	for(int i=0; i<strlen(newstr); i++){
		if( newstr[i] == '\\' ){
			newstr[i] = '/';
		}
	}

	return newstr;
}
#endif

==> OpenXOPS-tags_v1.030-20160111-r189.tar.gz: OpenXOPS-tags_v1.030-20160111-r189/v1.030-20160111/window.cpp <==
//! @file window.cpp
//! @brief CEBhE̍쐬E

//--------------------------------------------------------------------------------
// 
// OpenXOPS
// Copyright (c) 2014-2016, OpenXOPS Project / [-_-;](mikan) All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice, 
//   this list of conditions and the following disclaimer in the documentation 
//   and/or other materials provided with the distribution.
// * Neither the name of the OpenXOPS Project nor the names of its contributors 
//   may be used to endorse or promote products derived from this software 
//   without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL OpenXOPS Project BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//--------------------------------------------------------------------------------

#include "window.h"

//! @brief RXgN^
WindowControl::WindowControl()
{
	hInstance = 0;
	nCmdShow = 0;
	hWnd = NULL;
}

//! @brief fBXgN^
WindowControl::~WindowControl()
{
	//
}

//! @brief AvP[V̏ݒ
//! @param in_hInstance CX^X nh
//! @param in_nCmdShow EBhE̕\
void WindowControl::SetParam(HINSTANCE in_hInstance, int in_nCmdShow)
{
	hInstance = in_hInstance;
	nCmdShow = in_nCmdShow;
}

//! @brief CEBhE쐬
//! @param title EBhE^Cg
//! @param width EBhE̕
//! @param height EBhE̍
//! @param fullscreen falseFEBhE\@trueFtXN[p\
//! @return Ftrue@sFfalse
//! @warning SetParam()֐Őݒ肵ĂKv܂B
bool WindowControl::InitWindow(char* title, int width, int height, bool fullscreen)
{
	WNDCLASS wc;
	int x, y;
	RECT Rect;
	DWORD dwStyle;

	//EBhENX̓o^
	wc.style		= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc	= WindowProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInstance;
	wc.hIcon		= LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(MAINICON));
	wc.hCursor		= NULL;
	wc.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName	= NULL;
	wc.lpszClassName	= "MainWindow";
	if( !RegisterClass(&wc) ){
		return false;
	}

	if( fullscreen == false ){
		dwStyle = WS_OVERLAPPED | WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX;
	}
	else{
		dwStyle = WS_POPUP;
	}

	//EBhETCYvZ
	Rect.left = 0;
	Rect.top = 0;
	Rect.right = width;
	Rect.bottom = height;
	AdjustWindowRect(&Rect, dwStyle, FALSE);
	width = Rect.right - Rect.left;
	height = Rect.bottom - Rect.top;

	//\ʒu
	x = 0;
	y = 0;

	/*
	//EChE[hȂAEChEʂ̒Sɔzu
	if( fullscreen == false ){
		SystemParametersInfo(SPI_GETWORKAREA, 0, &Rect, 0);
		x = (Rect.right - width)/2;
		y = (Rect.bottom - height)/2;
	}
	*/

	//EBhE쐬
	hWnd = CreateWindow( "MainWindow", title, dwStyle, x, y, width, height, NULL, NULL, hInstance, NULL );
	if( hWnd == NULL ){
		return false;
	}

	//\
	ShowWindow( hWnd, nCmdShow );

	return true;
}

//! @brief CEBhẼEBhEvV[W
LRESULT WINAPI WindowControl::WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg ){
		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//! @brief EBhEnh擾
//! @return EBhEnh
HWND WindowControl::GethWnd()
{
	return hWnd;
}

//! @brief EChEbZ[WiC[vj
//! @return AChԁF0@Q[̃CsF1@EChEF-1
//! @attention {֐u1vԂꂽꍇɌAQ[̃CsĂB
//! @attention {֐u-1vԂꂽꍇAe\[XC^[tF[XAQ[IĂB
int WindowControl::CheckMainLoop()
{
	MSG msg = {0};

	if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) ){
		//EChEbZ[WĂ

		TranslateMessage( &msg );
		DispatchMessage( &msg );

		if( msg.message == WM_QUIT ){
			return -1;
		}
		return 0;
	}
	else if( GetActiveWindow() == hWnd ){
		//Q[̃t[
		return 1;
	}
	//else{
		//EChEbZ[W܂ő҂
		WaitMessage();
		return 0;
	//}
}

//! @brief G[bZ[W\
//! @param *str bZ[W
void WindowControl::ErrorInfo(char *str)
{
	MessageBox(hWnd, str, "error", MB_OK);
}

//! @brief EBhE悤ɗv
void WindowControl::CloseWindow()
{
	PostMessage(hWnd, WM_CLOSE, 0L, 0L);
}

//! @brief fpsiFrames Per SecondFt[[gjvZ
//! @param getcnt fps擾it[Pʁj
//! @return fps
float GetFps(int getcnt)
{
	static unsigned int ptimeg = 0;
	unsigned int nowtime;
	static float pfps = 0.0f;
	static int cnt = 0;

	if( cnt == 0 ){
		ptimeg = GetTimeMS();
	}
	if( cnt == getcnt ){
		nowtime = GetTimeMS();
		pfps = 1000.0f / ((nowtime - ptimeg)/getcnt);
		cnt = 0;
	}
	else{
		cnt += 1;
	}

	return pfps;
}

//! @brief fpsiFrames Per SecondFt[[gj
//! @return {Ftrue@@{Ffalse
bool ControlFps()
{
	static unsigned int ptimec = 0;
	unsigned int nowtime;
	int waittime;

	nowtime = GetTimeMS();
	waittime = GAMEFRAMEMS - (nowtime - ptimec);
	if( waittime > 0 ){
		Sleep(waittime);
		ptimec = GetTimeMS();
		return true;
	}

	ptimec = nowtime;
	return false;
}

//! @brief ~bPʂԂ
//! @return ~b
unsigned int GetTimeMS()
{
	unsigned int time;

	timeBeginPeriod(1);
	time = timeGetTime();
	timeEndPeriod(1);

	return time;
}

//! @brief ɂ镶擾
//! @param str 󂯎|C^
void GetTimeName(char *str)
{
	time_t timer;
	struct tm *local;

	timer = time(NULL);
	local = localtime(&timer);

	sprintf(str, "%04d%02d%02d%02d%02d%02d", local->tm_year + 1900, local->tm_mon + 1, local->tm_mday, local->tm_hour, local->tm_min, local->tm_sec);
}

//! @brief 
void InitRand()
{
	srand(GetTimeMS());
}

//! @brief _ȐlԂ
//! @param num ͈
//! @return 0`num-1
int GetRand(int num)
{
	return rand()%num;

	//return rand() / (RAND_MAX/num);

	//{XOPS̃ASYH
	//static int memory = GetTimeMS();
	//int x;
	//memory = memory * 214013 + 2745024;
	//x = memory >> 16;
	//x = x & 0x00007FFF;
	//return x%num;
}

#ifdef PATH_DELIMITER_SLASH
//! @brief pX؂蕶ϊ
//! @param *str t@CpX̃|C^
//! @return Vt@CpX̃|C^
//! @note '\''/'֒u܂B
char* ChangePathDelimiter(char *str)
{
	static char newstr[MAX_PATH];
	strcpy(newstr, str);

	for(int i=0; i<strlen(newstr); i++){
		if( newstr[i] == '\\' ){
			newstr[i] = '/';
		}
	}

	return newstr;
}
#endif
